@@ -182,6 +182,11 @@ JXG.extend(
             r0 = Type.evaluate(r);
             r = this.view.intersectionLineCube(p, d, r0);
 
+            // Check if r is infinite. This happens
+            // if this.vec is the zero vector.
+            if (Math.abs(r) === Infinity) {
+                r = 0;
+            }
             return [
                 p[0] + d[0] * r,
                 p[1] + d[1] * r,
@@ -300,7 +305,8 @@ JXG.extend(
             c3d[1] *= 0.5;
             c3d[2] *= 0.5;
             c3d[3] *= 0.5;
-            this.zIndex = Mat.matVecMult(this.view.matrix3DRotShift, c3d)[3];
+            // this.zIndex = Mat.matVecMult(this.view.matrix3DRotShift, c3d)[3];
+            this.zIndex = Mat.innerProduct(this.view.matrix3DRotShift[3], c3d);
 
             return this;
         }
@@ -642,7 +648,6 @@ JXG.createLine3D = function (board, parents, attributes) {
     } else {
         // Line defined by point, direction and range
 
-
         // Directions are handled as arrays of length 4, i.e. with homogeneous coordinates.
         if (base !== null) {
             point = Type.providePoints3D(view, [[0, 0, 0]], attributes, 'line3d', ['point'])[0];
