@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2024
+    Copyright 2008-2025
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -135,9 +135,9 @@ JXG.extend(
             var dx, dy, r, type, prec, c, v, p, dot,
                 len = this.transformations.length;
 
-            if (Type.isObject(Type.evaluate(this.visProp.precision))) {
+            if (Type.isObject(this.evalVisProp('precision'))) {
                 type = this.board._inputDevice;
-                prec = Type.evaluate(this.visProp.precision[type]);
+                prec = this.evalVisProp('precision.' + type);
             } else {
                 // 'inherit'
                 prec = this.board.options.precision.hasPoint;
@@ -329,8 +329,7 @@ JXG.extend(
 );
 
 /**
- * @class This element is used to provide a constructor for arbitrary content in
- * an SVG foreignObject container.
+ * @class Display any HTML content in an SVG foreignObject container - even below other elements.
  * <p>
  * Instead of board.create('foreignobject') the shortcut board.create('fo') may be used.
  *
