@@ -60,8 +60,6 @@ var priv = {
  * @class This element is used to provide a constructor for special texts containing a
  * HTML form input element.
  * For this element, the attribute "display" has to have the value 'html' (which is the default).
- * If the width of element is set with the attribute "cssStyle", the width of the
- * label must be added because cssStyle affects the surrounding div element.
  *
  * <p><b>Setting a CSS class:</b> The attribute <tt>cssClass</tt> affects the HTML div element that contains the input element. To change the CSS properties of the HTML input element a selector of the form
  * <tt>.myinput > input { ... }</tt> has to be used. See the analog example for buttons:
@@ -191,6 +189,21 @@ var priv = {
  *
  * </script><pre>
  *
+ * @example
+ * // change the width of an input field
+ *  let s = board.create('slider', [[-3, 3], [2, 3], [50, 100, 300]]);
+ *  let inp = board.create('input', [-6, 1, 'Math.sin(x)*x', 'f(x)='],{cssStyle:()=>'width:'+s.Value()+'px'});
+ *
+ * </pre><div id="JXG51c4d78b-a7ad-4c34-a983-b3ddae6192d7-1" class="jxgbox" style="width: 300px; height: 300px;"></div>
+ * <script type="text/javascript">
+ *     (function() {
+ *         var board = JXG.JSXGraph.initBoard('JXG51c4d78b-a7ad-4c34-a983-b3ddae6192d7-1',
+ *             {boundingbox: [-8, 8, 8,-8], axis: true, showcopyright: false, shownavigation: false});
+ *  let s = board.create('slider', [[-3, 3], [2, 3], [50, 100, 300]]);
+ *  let inp = board.create('input', [-6, 1, 'Math.sin(x)*x', 'f(x)='],{cssStyle:()=>'width:'+s.Value()+'px'});
+ *     })();
+ *
+ * </script><pre>
  *
  * @example
  *   Apply CSS classes to label and input tag
@@ -270,7 +283,7 @@ JXG.createInput = function (board, parents, attributes) {
     t.rendNodeTag.disabled = !!attr.disabled;
     t.rendNodeLabel.id = t.rendNode.id + "_label";
     t.rendNodeInput.id = t.rendNode.id + "_input";
-    t.rendNodeInput.setAttribute("aria-labelledby",t.rendNodeLabel.id);
+    t.rendNodeInput.setAttribute("aria-labelledby", t.rendNodeLabel.id);
 
     // 2. Set parents[3] (string|function) as label of the input element.
     // abstract.js selects the correct DOM element for the update
@@ -278,6 +291,18 @@ JXG.createInput = function (board, parents, attributes) {
 
     t._value = parents[2];
 
+    // 3.  capture keydown events on the input, and do not let them propagate.  The problem is that
+    // elevation controls on view3D use left and right, so editing the input triggers 3D pan.
+    t.rendNodeInput.addEventListener("keydown", (event) => {
+        // only trap left-and-right in case user wants input editing events
+        if (event.key === 'ArrowLeft' || event.key === 'ArrowRight') {
+            event.stopPropagation();
+        }
+    });
+
+
+
+
     /**
     * @class
     * @ignore
