@@ -1016,9 +1016,20 @@ JXG.extend(
          * Shows a small copyright notice in the top left corner of the board.
          * @param {String} str The copyright notice itself
          * @param {Number} fontsize Size of the font the copyright notice is written in
+         * @see JXG.AbstractRenderer#displayLogo
+         * @see Text#fontSize
          */
         displayCopyright: function (str, fontsize) { /* stub */ },
 
+        /**
+         * Shows a small JSXGraph logo in the top left corner of the board.
+         * @param {String} str The data-URL of the logo
+         * @param {Number} fontsize Size of the font the copyright notice is written in
+         * @see JXG.AbstractRenderer#displayCopyright
+         * @see Text#fontSize
+         */
+        displayLogo: function (str, fontsize) { /* stub */ },
+
         /**
          * An internal text is a {@link JXG.Text} element which is drawn using only
          * the given renderer but no HTML. This method is only a stub, the drawing
@@ -1226,7 +1237,7 @@ JXG.extend(
                         el.htmlStr = content;
 
                         if (el.evalVisProp('usemathjax')) {
-                            // Typesetting directly might not work because mathjax was not loaded completely
+                            // Typesetting directly might not work because MathJax was not loaded completely
                             try {
                                 if (MathJax.typeset) {
                                     // Version 3
@@ -1351,7 +1362,7 @@ JXG.extend(
         },
 
         /**
-         * Updates font-size, color and opacity propertiey and CSS style properties of a {@link JXG.Text} node.
+         * Updates font-size, color and opacity properties and CSS style properties of a {@link JXG.Text} node.
          * This function is also called by highlight() and nohighlight().
          * @param {JXG.Text} el Reference to the {@link JXG.Text} object, that has to be updated.
          * @param {Boolean} doHighlight
@@ -1406,19 +1417,19 @@ JXG.extend(
                         (doHighlight ? 'highlight' : '') + styleList[style]
                     );
                     // Set the CSS style properties - without deleting other properties
-                    if (cssString !== "" && el.visPropOld[styleList[style]] !== cssString) {
-                        cssList = this._css2js(cssString);
-                        for (node = 0; node < lenN; node++) {
-                            if (Type.exists(el[nodeList[node]])) {
+                    for (node = 0; node < lenN; node++) {
+                        if (Type.exists(el[nodeList[node]])) {
+                            if (cssString !== "" && el.visPropOld[styleList[style] + '_' + node] !== cssString) {
+                                cssList = this._css2js(cssString);
                                 for (prop in cssList) {
                                     if (cssList.hasOwnProperty(prop)) {
-                                        el[nodeList[node]].style[cssList[prop].key] =
-                                            cssList[prop].val;
+                                        el[nodeList[node]].style[cssList[prop].key] = cssList[prop].val;
                                     }
                                 }
+                                el.visPropOld[styleList[style] + '_' + node] = cssString;
                             }
                         }
-                        el.visPropOld[styleList[style]] = cssString;
+                        // el.visPropOld[styleList[style]] = cssString;
                     }
                 }
 
