@@ -203,7 +203,7 @@ JXG.SVGRenderer = function (container, dim) {
 
     /**
      * Combine arguments to a string, joined by empty string.
-     * Masks the container id with CSS.escape.
+     * The container id needs to be escaped, as it may contain URI-unsafe characters
      *
      * @params {String} str variable number of strings
      * @returns String
@@ -218,8 +218,8 @@ JXG.SVGRenderer = function (container, dim) {
         // ES6 would be [...arguments].join()
         var str = Array.prototype.slice.call(arguments).join('');
         // Mask special symbols like '/' and '\' in id
-        if (Type.exists(CSS) && Type.exists(CSS.escape)) {
-            str = CSS.escape(str);
+        if (Type.exists(encodeURIComponent)) {
+            str = encodeURIComponent(str);
         }
         return str;
     };
@@ -605,18 +605,43 @@ JXG.extend(
 
         // Already documented in JXG.AbstractRenderer
         displayCopyright: function (str, fontsize) {
-            var node = this.createPrim("text", 'licenseText'),
-                t;
-            node.setAttributeNS(null, 'x', '20px');
-            node.setAttributeNS(null, 'y', 2 + fontsize + 'px');
+            var node, t,
+                x = 4 + 1.8 * fontsize,
+                y = 6 + fontsize,
+                alpha = 0.2;
+
+            node = this.createPrim("text", 'licenseText');
+            node.setAttributeNS(null, 'x', x + 'px');
+            node.setAttributeNS(null, 'y', y + 'px');
             node.setAttributeNS(null, 'style', 'font-family:Arial,Helvetica,sans-serif; font-size:' +
-                fontsize + 'px; fill:#356AA0;  opacity:0.3;');
+                fontsize + 'px; opacity:' + alpha + ';');
+                // fill:#356AA0;
+            node.setAttributeNS(null, 'aria-hidden', 'true');
+
             t = this.container.ownerDocument.createTextNode(str);
-            node.setAttributeNS(null, 'aria-hidden', 'true');  // should NEVER be in screen reader
             node.appendChild(t);
             this.appendChildPrim(node, 0);
         },
 
+        // Already documented in JXG.AbstractRenderer
+        displayLogo: function (str, fontsize) {
+            var node,
+                s = 1.5 * fontsize,
+                alpha = 0.2;
+
+            node = this.createPrim("image", 'licenseLogo');
+            node.setAttributeNS(null, 'x', '5px');
+            node.setAttributeNS(null, 'y', '5px');
+            node.setAttributeNS(null, 'width', s + 'px');
+            node.setAttributeNS(null, 'height', s + 'px');
+            node.setAttributeNS(null, "preserveAspectRatio", "none");
+            node.setAttributeNS(null, 'style', 'opacity:' + alpha + ';');
+            node.setAttributeNS(null, 'aria-hidden', 'true');
+
+            node.setAttributeNS(this.xlinkNamespace, "xlink:href", str);
+            this.appendChildPrim(node, 0);
+        },
+
         // Already documented in JXG.AbstractRenderer
         drawInternalText: function (el) {
             var node = this.createPrim("text", el.id);
@@ -1468,7 +1493,7 @@ JXG.extend(
                     node = rendNode;
                 }
 
-                if (c !== "none") {
+                if (c !== "none" && c !== "" && c !== false) {
                     this._setAttribute(function () {
                         node.setAttributeNS(null, "fill", c);
                     }, el.visPropOld.fillcolor);
